/*-
 * Copyright (c) 2004 Michael Bushkov <bushman@rsu.ru>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <pthread.h>
#include <stdio.h>

#define DECLARE_TSS(x,ytype,shouldinit,y) \
	static pthread_key_t _##x##_tss_key=(pthread_key_t)NULL; \
	\
	static void _##x##_tss_init(void)\
	{\
		ytype * initvalue;\
		\
		initvalue=(ytype *)malloc(sizeof(ytype)); \
		memset(initvalue,0,sizeof(ytype));\
		if (shouldinit==1) \
			*initvalue= y; \
		\
		pthread_setspecific(_##x##_tss_key,(void *)initvalue); \
	}\
	\
	static ytype *_##x##_tss_get(void)\
	{\
		if (((void *)pthread_getspecific(_##x##_tss_key))==NULL)\
			_##x##_tss_init();\
		\
		return pthread_getspecific(_##x##_tss_key);\
	}

#define DECLARE_TSS_WITH_INIT(x,ytype,fn) \
	static pthread_key_t _##x##_tss_key=(pthread_key_t)NULL; \
	\
	static void _##x##_tss_init(void)\
	{\
		ytype * initvalue=(ytype *)malloc(sizeof(ytype)); \
		\
		*initvalue= fn(); \
		pthread_setspecific(_##x##_tss_key,(void *)initvalue); \
	}\
	\
	static ytype *_##x##_tss_get(void)\
	{\
		if (pthread_getspecific(_##x##_tss_key)==NULL)\
			_##x##_tss_init();\
		\
		return pthread_getspecific(_##x##_tss_key);\
	}

#define INIT_TSS_KEY(x) pthread_key_create(&_##x##_tss_key,NULL);
#define GET_TSS(x) (*_##x##_tss_get())
#define GET_TSS_PTR(x) (_##x##_tss_get())
